package algebra

import org.scalatest._
import testutil.PendingIfUnimplemented

class MonoidsSpec extends FlatSpec with Matchers with AppendedClues with PendingIfUnimplemented {
  "The addditive Int monoid" should "have the right zero element" in {
    Monoids.intAddition.zero shouldBe 0
  }
  it should "combine multiple values correctly" in {
    Monoids.intAddition.op(1, 2) shouldBe 3
  }

  "The multiplicative Int monoid" should "have the right zero element" in {
    Monoids.intMultiplication.zero shouldBe 1
  }
  it should "combine multiple values correctly" in {
    Monoids.intMultiplication.op(1, 2) shouldBe 2
  }

  "The boolean or monoid" should "have the right zero element" in {
    Monoids.booleanOr.zero shouldBe false
  }
  it should "combine multiple values correctly" in {
    Monoids.booleanOr.op(false, true) shouldBe true
    Monoids.booleanOr.op(false, false) shouldBe false
  }

  "The boolean and monoid" should "have the right zero element" in {
    Monoids.booleanAnd.zero shouldBe true
  }
  it should "combine multiple values correctly" in {
    Monoids.booleanAnd.op(false, true) shouldBe false
    Monoids.booleanAnd.op(false, false) shouldBe false
    Monoids.booleanAnd.op(true, true) shouldBe true
  }

  "The option monoid" should "always return the non-empty option" in {
    Monoids.optionMonoid[Int].op(Some(3), None) shouldBe Some(3)
    Monoids.optionMonoid[Int].op(None, Some(3)) shouldBe Some(3)
  }

  "The endofunction monoid" should "always give the right zero element" in {
    Monoids.endoMonoid[Int].zero(3) shouldBe 3
  }

  it should "combine the functions one way or the other" in {
    Monoids.endoMonoid[Int].op(_ / 4, _ * 2)(4) shouldBe 2
  }

  "the foldMap implementation" should "give the right result" in {
    val l = List("1", "2", "3")

    Monoids.foldMap(l, new Monoid[Int] {
      def zero = 10
      def op(a: Int, b: Int) = a + b
    })(_.toInt) shouldBe 16
  }

  "the balanced foldMap implementation" should "give the right result" in {
    val l = Vector("1", "2", "3")

    Monoids.foldMapBalanced(l, new Monoid[Int] {
      def zero = 0
      def op(a: Int, b: Int) = a + b
    })(_.toInt) shouldBe 6
  }

  "the bag function" should "count correctly" in {
    val words = "a rose is a rose".split(" ")

    Monoids.bag(words) shouldBe Map("a" -> 2, "rose" -> 2, "is" -> 1)
  }
}
