package errors

import org.scalatest._
import testutil.PendingIfUnimplemented

class OptionSpec extends FlatSpec with Matchers with AppendedClues with PendingIfUnimplemented {
  "An option" should "have a map function which transforms it's content" in {
    Some(3).map(_ + 1) shouldBe Some(4)
  }

  it should "have a getOrElse method which returns its content or a default value when it's empty" in {
    Some(3).getOrElse(4) shouldBe 3
    None.getOrElse(4) shouldBe 4
  }


  it should "have a flatMap which works according to the monad laws" in {
    Some(3).flatMap(x => Some(x + 3)) shouldBe Some(6)
    (None: Option[Int]).flatMap(x => Some(x + 3)) shouldBe None
  }

  it should "have a filter method which removes its content according to a give predicate" in {
    Some(3).filter(_ % 2 == 0) shouldBe None
    Some(4).filter(_ % 2 == 0) shouldBe Some(4)
  }

  it should "have a sequence method" in {
    Option.sequence(List.empty[Option[Int]]) shouldBe Some(Nil)
    Option.sequence(List(Some(1), Some(2))) shouldBe Some(List(1,2))
    Option.sequence(List(Some(1), None)) shouldBe None
  }
}

